@@ -13,7 +13,7 @@ AnyEvent::Handle - non-blocking I/O on streaming handles via AnyEvent
       fh => \*STDIN,
       on_error => sub {
          my ($hdl, $fatal, $msg) = @_;
-         warn "got error $msg\n";
+         AE::log warn => "got error $msg\n";
          $hdl->destroy;
          $cv->send;
       };
@@ -24,7 +24,7 @@ AnyEvent::Handle - non-blocking I/O on streaming handles via AnyEvent
    # read the response line
    $hdl->push_read (line => sub {
       my ($hdl, $line) = @_;
-      warn "got line <$line>\n";
+      AE::log warn => "got line <$line>\n";
       $cv->send;
    });
 
@@ -116,10 +116,10 @@ appropriate circumstances:
 =item on_prepare => $cb->($handle)
 
 This (rarely used) callback is called before a new connection is
-attempted, but after the file handle has been created. It could be used to
-prepare the file handle with parameters required for the actual connect
-(as opposed to settings that can be changed when the connection is already
-established).
+attempted, but after the file handle has been created (you can access that
+file handle via C<< $handle->{fh} >>). It could be used to prepare the
+file handle with parameters required for the actual connect (as opposed to
+settings that can be changed when the connection is already established).
 
 The return value of this callback should be the connect timeout value in
 seconds (or C<0>, or C<undef>, or the empty list, to indicate that the
@@ -249,22 +249,27 @@ file handle (or a call to C<timeout_reset>), the C<on_timeout> callback
 will be invoked (and if that one is missing, a non-fatal C<ETIMEDOUT>
 error will be raised).
 
-There are three variants of the timeouts that work independently
-of each other, for both read and write, just read, and just write:
+There are three variants of the timeouts that work independently of each
+other, for both read and write (triggered when nothing was read I<OR>
+written), just read (triggered when nothing was read), and just write:
 C<timeout>, C<rtimeout> and C<wtimeout>, with corresponding callbacks
 C<on_timeout>, C<on_rtimeout> and C<on_wtimeout>, and reset functions
 C<timeout_reset>, C<rtimeout_reset>, and C<wtimeout_reset>.
 
-Note that timeout processing is active even when you do not have
-any outstanding read or write requests: If you plan to keep the connection
-idle then you should disable the timeout temporarily or ignore the timeout
-in the C<on_timeout> callback, in which case AnyEvent::Handle will simply
-restart the timeout.
+Note that timeout processing is active even when you do not have any
+outstanding read or write requests: If you plan to keep the connection
+idle then you should disable the timeout temporarily or ignore the
+timeout in the corresponding C<on_timeout> callback, in which case
+AnyEvent::Handle will simply restart the timeout.
 
-Zero (the default) disables this timeout.
+Zero (the default) disables the corresponding timeout.
 
 =item on_timeout => $cb->($handle)
 
+=item on_rtimeout => $cb->($handle)
+
+=item on_wtimeout => $cb->($handle)
+
 Called whenever the inactivity timeout passes. If you return from this
 callback, then the timeout will be reset as if some activity had happened,
 so this condition is not fatal in any way.
@@ -281,6 +286,21 @@ be configured to accept only so-and-so much data that it cannot act on
 amount of data without a callback ever being called as long as the line
 isn't finished).
 
+=item wbuf_max => <bytes>
+
+If defined, then a fatal error will be raised (with C<$!> set to C<ENOSPC>)
+when the write buffer ever (strictly) exceeds this size. This is useful to
+avoid some forms of denial-of-service attacks.
+
+Although the units of this parameter is bytes, this is the I<raw> number
+of bytes not yet accepted by the kernel. This can make a difference when
+you e.g. use TLS, as TLS typically makes your write data larger (but it
+can also make it smaller due to compression).
+
+As an example of when this limit is useful, take a chat server that sends
+chat messages to a client. If the client does not read those in a timely
+manner then the send buffer in the server would grow unbounded.
+
 =item autocork => <boolean>
 
 When disabled (the default), C<push_write> will try to immediately
@@ -341,10 +361,11 @@ from most attacks.
 
 =item read_size => <bytes>
 
-The initial read block size, the number of bytes this module will try to
-read during each loop iteration. Each handle object will consume at least
-this amount of memory for the read buffer as well, so when handling many
-connections requirements). See also C<max_read_size>. Default: C<2048>.
+The initial read block size, the number of bytes this module will try
+to read during each loop iteration. Each handle object will consume
+at least this amount of memory for the read buffer as well, so when
+handling many connections watch out for memory requirements). See also
+C<max_read_size>. Default: C<2048>.
 
 =item max_read_size => <bytes>
 
@@ -523,7 +544,7 @@ sub new {
                   } else {
                      if ($self->{on_connect_error}) {
                         $self->{on_connect_error}($self, "$!");
-                        $self->destroy;
+                        $self->destroy if $self;
                      } else {
                         $self->_error ($!, 1);
                      }
@@ -533,7 +554,7 @@ sub new {
                   local $self->{fh} = $_[0];
 
                   $self->{on_prepare}
-                     ?  $self->{on_prepare}->($self)
+                     ? $self->{on_prepare}->($self)
                      : ()
                }
             );
@@ -742,12 +763,20 @@ sub on_stoptls {
 
 Configures the C<rbuf_max> setting (C<undef> disables it).
 
+=item $handle->wbuf_max ($max_octets)
+
+Configures the C<wbuf_max> setting (C<undef> disables it).
+
 =cut
 
 sub rbuf_max {
    $_[0]{rbuf_max} = $_[1];
 }
 
+sub wbuf_max {
+   $_[0]{wbuf_max} = $_[1];
+}
+
 #############################################################################
 
 =item $handle->timeout ($seconds)
@@ -758,6 +787,9 @@ sub rbuf_max {
 
 Configures (or disables) the inactivity timeout.
 
+The timeout will be checked instantly, so this method might destroy the
+handle before it returns.
+
 =item $handle->timeout_reset
 
 =item $handle->rtimeout_reset
@@ -875,9 +907,9 @@ sub on_drain {
 
 =item $handle->push_write ($data)
 
-Queues the given scalar to be written. You can push as much data as you
-want (only limited by the available memory), as C<AnyEvent::Handle>
-buffers it independently of the kernel.
+Queues the given scalar to be written. You can push as much data as
+you want (only limited by the available memory and C<wbuf_max>), as
+C<AnyEvent::Handle> buffers it independently of the kernel.
 
 This method may invoke callbacks (and therefore the handle might be
 destroyed after it returns).
@@ -915,6 +947,13 @@ sub _drain_wbuf {
       # if still data left in wbuf, we need to poll
       $self->{_ww} = AE::io $self->{fh}, 1, $cb
          if length $self->{wbuf};
+
+      if (
+         defined $self->{wbuf_max}
+         && $self->{wbuf_max} < length $self->{wbuf}
+      ) {
+         $self->_error (Errno::ENOSPC, 1), return;
+      }
    };
 }
 
@@ -1059,7 +1098,7 @@ C<on_drain> handler by a callback that shuts down the socket (and set
 C<low_water_mark> to C<0>). This method is a shorthand for just that, and
 replaces the C<on_drain> callback with:
 
-   sub { shutdown $_[0]{fh}, 1 }    # for push_shutdown
+   sub { shutdown $_[0]{fh}, 1 }
 
 This simply shuts down the write side and signals an EOF condition to the
 the peer.
@@ -1395,7 +1434,7 @@ data.
 Example: read 2 bytes.
 
    $handle->push_read (chunk => 2, sub {
-      warn "yay ", unpack "H*", $_[1];
+      AE::log debug => "yay " . unpack "H*", $_[1];
    });
 
 =cut
@@ -1507,13 +1546,13 @@ register_read_type regex => sub {
       # accept
       if ($$rbuf =~ $accept) {
          $data .= substr $$rbuf, 0, $+[0], "";
-         $cb->($self, $data);
+         $cb->($_[0], $data);
          return 1;
       }
       
       # reject
       if ($reject && $$rbuf =~ $reject) {
-         $self->_error (Errno::EBADMSG);
+         $_[0]->_error (Errno::EBADMSG);
       }
 
       # skip
@@ -1539,20 +1578,20 @@ register_read_type netstring => sub {
    sub {
       unless ($_[0]{rbuf} =~ s/^(0|[1-9][0-9]*)://) {
          if ($_[0]{rbuf} =~ /[^0-9]/) {
-            $self->_error (Errno::EBADMSG);
+            $_[0]->_error (Errno::EBADMSG);
          }
          return;
       }
 
       my $len = $1;
 
-      $self->unshift_read (chunk => $len, sub {
+      $_[0]->unshift_read (chunk => $len, sub {
          my $string = $_[1];
          $_[0]->unshift_read (chunk => 1, sub {
             if ($_[1] eq ",") {
                $cb->($_[0], $string);
             } else {
-               $self->_error (Errno::EBADMSG);
+               $_[0]->_error (Errno::EBADMSG);
             }
          });
       });
@@ -1635,26 +1674,26 @@ register_read_type json => sub {
    my $rbuf = \$self->{rbuf};
 
    sub {
-      my $ref = eval { $json->incr_parse ($self->{rbuf}) };
+      my $ref = eval { $json->incr_parse ($_[0]{rbuf}) };
 
       if ($ref) {
-         $self->{rbuf} = $json->incr_text;
+         $_[0]{rbuf} = $json->incr_text;
          $json->incr_text = "";
-         $cb->($self, $ref);
+         $cb->($_[0], $ref);
 
          1
       } elsif ($@) {
          # error case
          $json->incr_skip;
 
-         $self->{rbuf} = $json->incr_text;
+         $_[0]{rbuf} = $json->incr_text;
          $json->incr_text = "";
 
-         $self->_error (Errno::EBADMSG);
+         $_[0]->_error (Errno::EBADMSG);
 
          ()
       } else {
-         $self->{rbuf} = "";
+         $_[0]{rbuf} = "";
 
          ()
       }
@@ -1697,7 +1736,7 @@ register_read_type storable => sub {
             if (my $ref = eval { Storable::thaw ($_[1]) }) {
                $cb->($_[0], $ref);
             } else {
-               $self->_error (Errno::EBADMSG);
+               $_[0]->_error (Errno::EBADMSG);
             }
          });
       }
@@ -1745,15 +1784,24 @@ you change the C<on_read> callback or push/unshift a read callback, and it
 will automatically C<stop_read> for you when neither C<on_read> is set nor
 there are any read requests in the queue.
 
-These methods will have no effect when in TLS mode (as TLS doesn't support
-half-duplex connections).
+In older versions of this module (<= 5.3), these methods had no effect,
+as TLS does not support half-duplex connections. In current versions they
+work as expected, as this behaviour is required to avoid certain resource
+attacks, where the program would be forced to read (and buffer) arbitrary
+amounts of data before being able to send some data. The drawback is that
+some readings of the the SSL/TLS specifications basically require this
+attack to be working, as SSL/TLS implementations might stall sending data
+during a rehandshake.
+
+As a guideline, during the initial handshake, you should not stop reading,
+and as a client, it might cause problems, depending on your applciation.
 
 =cut
 
 sub stop_read {
    my ($self) = @_;
 
-   delete $self->{_rw} unless $self->{tls};
+   delete $self->{_rw};
 }
 
 sub start_read {
@@ -1965,7 +2013,8 @@ sub starttls {
    $self->{_rbio} = Net::SSLeay::BIO_new (Net::SSLeay::BIO_s_mem ());
    $self->{_wbio} = Net::SSLeay::BIO_new (Net::SSLeay::BIO_s_mem ());
 
-   Net::SSLeay::BIO_write ($self->{_rbio}, delete $self->{rbuf});
+   Net::SSLeay::BIO_write ($self->{_rbio}, $self->{rbuf});
+   $self->{rbuf} = "";
 
    Net::SSLeay::set_bio ($tls, $self->{_rbio}, $self->{_wbio});
 
@@ -2013,6 +2062,19 @@ sub _freetls {
    delete @$self{qw(_rbio _wbio _tls_wbuf _on_starttls)};
 }
 
+=item $handle->resettls
+
+This rarely-used method simply resets and TLS state on the handle, usually
+causing data loss.
+
+One case where it may be useful is when you want to skip over the data in
+the stream but you are not interested in interpreting it, so data loss is
+no concern.
+
+=cut
+
+*resettls = \&_freetls;
+
 sub DESTROY {
    my ($self) = @_;
 
@@ -2247,6 +2309,10 @@ will be in C<$_[0]{rbuf}>:
       my $data = delete $_[0]{rbuf};
    });
 
+Note that this example removes the C<rbuf> member from the handle object,
+which is not normally allowed by the API. It is expressly permitted in
+this case only, as the handle object needs to be destroyed afterwards.
+
 The reason to use C<on_error> is that TCP connections, due to latencies
 and packets loss, might get closed quite violently with an error, when in
 fact all data has been received.
@@ -2266,7 +2332,7 @@ written to the socket:
 
    $handle->push_write (...);
    $handle->on_drain (sub {
-      warn "all data submitted to the kernel\n";
+      AE::log debug => "all data submitted to the kernel\n";
       undef $handle;
    });
 
